# -*- coding: utf-8 -*-

# windows: set VS90COMNTOOLS=%VS120COMNTOOLS% python build.py build --boost-dir=../../lib/boost_1.58_win64 --event-engine-include=../../../ --event-engine-lib=../../lib/win64/
# mac: python setup.py install --boost-dir=../../lib/boost_1.58_mac --event-engine-include=../../../ --event-engine-lib=../../lib/mac/
from distutils.version import StrictVersion
from setuptools import (
    Extension,
    find_packages,
    setup,
)
from sys import platform as _platform
import sys
import copy

BOOST_HOME = 'boost'
EVENT_ENGINE_INCLUDE ='eventengine/include'
EVENT_ENGINE_LIB ='eventengine/lib'

BOOST_HOME_CMD = '--boost-dir'
EVENT_ENGINE_INCLUDE_CMD = '--event-engine-include'
EVENT_ENGINE_LIB_CMD = '--event-engine-lib'

#sys.setdefaultencoding("UTF-8")

#module1 = Extension('keystone',
#		include_dirs = [
#		'/Users/RK/Desktop/shared_folder/project/keystone-strategy-engine/lib/boost_1.58_mac/include',
#		'/Users/RK/Desktop/shared_folder/project/keystone-strategy-engine/src/'],
#		libraries = ['boost_system', 'boost_thread', 'eventengine'],
#		library_dirs = [
#		'/Users/RK/Desktop/shared_folder/project/keystone-strategy-engine/lib/boost_1.58_mac/lib',
#		'/Users/RK/Desktop/shared_folder/project/keystone-strategy-engine/lib/mac/'],
#		extra_compile_args = ['-std=c++0x','-stdlib=libc++','-mmacosx-version-min=10.7'],
#		extra_link_args = ['-stdlib=libc++','-mmacosx-version-min=10.7'],
#		language = 'c++',
#		sources = ['kspython_interface.cpp'])

# module1 = Extension('keystone',
# 		include_dirs = [
# 		'/mnt/hgfs/project/keystone-strategy-engine/lib/boost_1.58_unix/include',
# 		'/mnt/hgfs/project/keystone-strategy-engine/src/'],
# 		libraries = ['eventengine', 'boost_system', 'boost_thread'],
# 		library_dirs = [
# 		'/mnt/hgfs/project/keystone-strategy-engine/lib/boost_1.58_unix/lib',
# 		'/mnt/hgfs/project/keystone-strategy-engine/lib/unix/'],
# 		extra_compile_args = ['-std=c++0x'],
# 		language = 'c++',
# 		sources = ['kspython_interface.cpp'])

#http://victorjabur.com/2011/06/05/compiling-python-2-7-modules-on-windows-32-and-64-using-msvc-2008-express/
def parse_option(arg, name):
    if arg.find(name) == 0:
        idx = arg.find(name + '=')
        if idx == -1:
            raise TypeError('Unknow option ' + arg)
        else:
            return arg[len(name)+1:].strip()
    else:
        return None

if __name__ == '__main__':
    args = copy.copy(sys.argv)
    for arg in args:
        # parse boost dir
        ret = parse_option(arg, BOOST_HOME_CMD)
        if ret is not None:
            BOOST_HOME = ret
            sys.argv.remove(arg)
            continue
        # parse event engine include
        ret = parse_option(arg, EVENT_ENGINE_INCLUDE_CMD)
        if ret is not None:
            EVENT_ENGINE_INCLUDE = ret
            sys.argv.remove(arg)
            continue
        # parse event engine include
        ret = parse_option(arg, EVENT_ENGINE_LIB_CMD)
        if ret is not None:
            EVENT_ENGINE_LIB = ret
            sys.argv.remove(arg)
            continue

    print("BOOST_HOME: " + BOOST_HOME)
    print("EVENT_ENGINE_INCLUDE: " + EVENT_ENGINE_INCLUDE)
    print("EVENT_ENGINE_LIB: " + EVENT_ENGINE_LIB)

    # compile and link args
    libraries = ['boost_system', 'boost_thread', 'eventengine']
    extra_compile_args = []
    extra_link_args = []
    if _platform == "linux" or _platform == "linux2":
       # linux
       extra_compile_args = ['-std=c++0x']
       extra_link_args =[]
    elif _platform == "darwin":
       # MAC OS X
       extra_compile_args = ['-std=c++0x','-stdlib=libc++','-Wno-unused-local-typedefs']
       extra_link_args = ['-stdlib=libc++']
    elif _platform == "win32":
       # Windows
       extra_compile_args = ['/EHsc']
       extra_link_args =[]
       libraries = ['eventengine']

    # c++ extension
    module1 = Extension('keystone.engine',
    		include_dirs = [
    		'./keystone/engine/',
    		BOOST_HOME + '/include',
            EVENT_ENGINE_INCLUDE],
    		libraries = libraries,
    		library_dirs = [
    		BOOST_HOME + '/lib',
    		EVENT_ENGINE_LIB],
    		extra_compile_args = extra_compile_args,
            extra_link_args = extra_link_args,
    		language = 'c++',
    		sources = ['./keystone/engine/kspython_interface.cpp'])

    # setup
    setup (name = 'keystone_sdk',
    		version = '0.0.1',
    		description = 'keystone python SDK for backtesting',
            packages=find_packages('.', include=['keystone', 'keystone.*']),
    		ext_modules = [module1])



# -*- coding: utf-8 -*-

from __future__ import print_function
from datetime import datetime
import pprint
import sys
import time
import zmq

from keystone.api import keystone_class, api_method
from keystone.coordinator import KSObserver, KSCoordinator
from keystone.app_client.session import Session
from keystone.order import KSOrderEventType
from keystone.py3compat import PY3, builtin_mod, iteritems
from keystone.utils.json_utils import json_clean, UTC_TIMESTAMP
import keystone.utils.zmq_utils as zmq_utils

if not PY3:
	from Queue import Queue
	import thread as _thread
else:
	from queue import Queue
	import _thread

class KSDrain(KSObserver):
	class DrainThreadStop:
		pass
	session = Session()
	context = zmq.Context()
	send_data = False
	def __init__(self, 
		strategy_manager, 
		drain_url='tcp://127.0.0.1:55558', 
		syn_url='tcp://127.0.0.1:55559', 
		debug=False):
		self.strategy_manager = strategy_manager
		self.drain_url = drain_url
		self.syn_url = syn_url
		self.debug = debug
		self.orders = []
		self.buffer = Queue()
		self.sock = zmq_utils.create_publisher_socket(self.drain_url, self.syn_url)

	def msg(self, dt):
		msg = {
		'datetime': dt,
		'portfolio_value': self.strategy_manager.portfolio.value(),
		'cash': self.strategy_manager.portfolio.cash(),
		'positions': [pos.to_dict() for pos in self.strategy_manager.portfolio.positions()],
		'orders': [order.to_dict() for order in self.orders],
		}
		msg.update(self.strategy_manager.analyzers[0].value())
		return msg

	def onDataEvent(self, dataEvent):
		# generate message
		msg = self.msg(dataEvent.time())
		if self.send_data:
			msg['data'] = dataEvent.getAllData()
		msg = json_clean(msg, datetime_format = UTC_TIMESTAMP)
		# clear orders
		self.orders = []
		# trigger send method
		self.send(msg)

	def onOrderEvent(self, orderEvent):
	    if orderEvent.type != KSOrderEventType.ACCEPTED and orderEvent.type != KSOrderEventType.CANCELLED:
	    	self.orders.append(orderEvent.txn)

	def send(self, msg):
		# self.buffer.put(msg, block = True)
		self.session.send(self.sock, 'drain', content = msg, ident = b'stream.drain')

	def sendMessage(self):
		count = 1
		while True:
			message = self.buffer.get(block = True)
			if isinstance(message, self.DrainThreadStop):
				return
			self.session.send(self.sock, 'drain', content = message, ident = b'stream.drain')
			# print "send %d"%(count)
			# count += 1

	def start(self):
		return
		_thread.start_new_thread(self.sendMessage, ())

	def wait(self):
		self.buffer.put(self.DrainThreadStop())



# -*- coding: utf-8 -*-

from __future__ import print_function
import random
import zmq
import sys
import time
import json
import importlib
import imp
import abc
import pytz
from pdb import Pdb
from datetime import datetime
from six import with_metaclass
from zmq.eventloop import ioloop, zmqstream
from zmq.eventloop.zmqstream import ZMQStream
import linecache
import multiprocessing

from keystone.app_client import zmq_utils 
from keystone.utils.json_utils import json_clean, UTC_TIMESTAMP
from keystone.app_client.datetime_breakpoint import DatetimeBreakpoint
from keystone.py3compat import safe_unicode, PY3
from keystone.universe import KSUniverse as Universe

if not PY3:
	from Queue import Queue
	import thread as _thread
else:
	from queue import Queue
	import _thread


KPDB_NOTIFY_REACH_BREAKPOINT = "kpdb_notify_reach_breakpoint"
KPDB_SET_BREAKPOINT_REPLY = "kpdb_set_breakpoint_reply"

KPDB_MSG_TYPE = [KPDB_NOTIFY_REACH_BREAKPOINT, KPDB_SET_BREAKPOINT_REPLY]

class KPdb(Pdb):
	def __init__(self, 
		filename,
		session, 
		notify_url, 
		syn_url, 
		control_url,
		datetime_break_func=None,
		datetime_break_cond=None,
		completekey='tab', stdin=None, stdout=None, skip=None):
		Pdb.__init__(self, completekey, stdin, stdout, skip)
		self.filename = filename
		self.stop_flag = False
		self.pause_flag = False
		self.lock = multiprocessing.Lock() 
		self.session = session
		self.init_notify_sock(notify_url, syn_url)
		self.init_control_sock(control_url)

		# set datetime break condition
		self.datetime_break_func = datetime_break_func
		self.datetime_break_cond = datetime_break_cond
		self.current_time_func = lambda: Universe.time
		if self.datetime_break_func is not None:
			self._break_at_function(self.datetime_break_func, cond=self.datetime_break_cond)

	def init_notify_sock(self, notify_url, syn_url):
		self.notify_socket = zmq_utils.create_publisher_socket(notify_url, syn_url)

	def init_control_sock(self, control_url):
		print("control_url: %s"%(control_url))
		control_socket = zmq.Context().socket(zmq.ROUTER)
		control_socket.bind(control_url)
		self.control_socket = control_socket
		# 改掉，不能再这里启动线程
		_thread.start_new_thread(self.handle_control_message, (control_socket,))

	def handle_control_message(self, socket, mode=zmq.NOBLOCK, content=True):
		"""Receive and unpack a message."""
		print("control_thread started")
		if isinstance(socket, ZMQStream):
			socket = socket.socket
		while 1:
			try:
			    msg_list = socket.recv_multipart(mode, copy=True)
			except zmq.ZMQError as e:
				if e.errno == zmq.EAGAIN:
					# We can convert EAGAIN to None as we know in this case
					# recv_multipart won't return None.
					continue
				else:
					raise
			try:
				print("==========================")
				print("recieve control message")
				print(msg_list)
				ident, reply = self.session.deserialize(msg_list, content=content)
				print("*********ident(%s), recieve %s"%(ident, reply), file=sys.stderr)
				self.set_pause()
				self.session.send(socket, safe_unicode('stop_reply'), content = {'result': 0}, ident = ident)
			except Exception as e:
				# TODO: handle it
				raise e

	def notify_frontend(self, msg_type, content):
		assert msg_type in KPDB_MSG_TYPE

		content = json_clean(content)
		msg = self.session.send(self.notify_socket, safe_unicode(msg_type), content = content, ident = b'stream.kpdb_notify')

	def construct_notify_message(self, frame):
		fname = self.canonic(frame.f_code.co_filename)
		lineno = frame.f_lineno
		if DatetimeBreakpoint.hit_datetime_breakpoint is None:
			content = json_clean(dict(type='line_breakpoint', filename=fname, lineno=lineno))
		else:
			content = json_clean(dict(
				type='datetime_breakpoint', 
				datetime=DatetimeBreakpoint.hit_datetime_breakpoint, 
				filename=fname, 
				lineno=lineno,
				datetime_format=UTC_TIMESTAMP))
		return content

	def trace_dispatch(self, frame, event, arg):
		with self.lock:
			self.stop_flag = False
		return Pdb.trace_dispatch(self, frame, event, arg)

	def forget(self):
		DatetimeBreakpoint.hit_datetime_breakpoint = None
		Pdb.forget(self)

	def set_pause(self):
		with self.lock:
			print("self.stop_flag = %d"%(self.stop_flag), file=sys.stderr)
			if not self.stop_flag:
				self.pause_flag = True

	# overide step_here,用于限制用户只能停留在当前文件中
	def stop_here(self, frame):
		fname = self.canonic(frame.f_code.co_filename)
		# print(self.frame_returning)
		# print(self.stopframe)
		# print(self.botframe)
		# print(self.stoplineno)
		# if self.botframe is not None:
		# 	print(self.botframe.f_code)
		if fname != self.filename:
			return False
		with self.lock:
			if self.stop_flag:
				return True
			elif self.pause_flag:
				self.stop_flag = True
				self.pause_flag = False # reset pause flag
				print("func stop_here, pause_flag=%d, file=%s, code=%s"%(self.pause_flag, fname, frame.f_code), file=sys.stderr)
				return True
			elif Pdb.stop_here(self,frame):
				self.stop_flag = True
				return True
			else:
				return False
		
	def user_return(self, frame, return_value):
		# construct notify message
		# content = self.construct_notify_message(frame)
		# notify frontend
		# self.notify_frontend(msg_type=KPDB_NOTIFY_REACH_BREAKPOINT, content=content)
		Pdb.user_return(self, frame, return_value)
		
	def user_call(self, frame, argument_list):
		# construct notify message
		# content = self.construct_notify_message(frame)
		# notify frontend
		# self.notify_frontend(msg_type=KPDB_NOTIFY_REACH_BREAKPOINT, content=content)
		Pdb.user_call(self, frame, argument_list)

	def user_line(self, frame):
		# construct notify message
		content = self.construct_notify_message(frame)
		# notify frontend
		self.notify_frontend(msg_type=KPDB_NOTIFY_REACH_BREAKPOINT, content=content)
		# call Pdb.user_line
		Pdb.user_line(self, frame)

	''' custom clear_all function without interaction with user '''
	def do_clear_all(self, arg):
		self.clear_all_breaks()
		DatetimeBreakpoint.clear()
		# restore datetime break condition
		if self.datetime_break_func is not None:
			self._break_at_function(self.datetime_break_func, cond=self.datetime_break_cond)

	''' override original do_clear function '''
	def do_clear(self, arg):
		if not arg:
			DatetimeBreakpoint.clear()
		Pdb.do_clear(self,arg)
		# restore datetime break condition
		if self.datetime_break_func is not None:
			self._break_at_function(self.datetime_break_func, cond=self.datetime_break_cond)

	def do_break_line(self, arg, temporary = 0):
		filename = None
		lineno = None
		colon = arg.rfind(':')
		if colon >= 0:
			filename = arg[:colon].rstrip()
			arg = arg[colon+1:].lstrip()
			try:
				lineno = int(arg)
				err = self._break_at_line(filename, lineno, temporary)
				if err:
					pass
					# print >>sys.stderr, err
					# return
			except ValueError as msg:
				print('***(KPdb) Bad lineno:', arg, file=sys.stderr)
				return
		else:
			print("***(KPdb) Bad arguments: ", arg, file=sys.stderr)

	def do_unbreak_line(self, arg, temporary = 0):
		filename = None
		lineno = None
		colon = arg.rfind(':')
		if colon >= 0:
			filename = arg[:colon].rstrip()
			arg = arg[colon+1:].lstrip()
			try:
				lineno = int(arg)
				err = self.clear_break(filename, lineno)
				if err:
					pass
					# print >>sys.stderr, err
					# return
			except ValueError as msg:
				print('***(KPdb) Bad lineno:', arg, file=sys.stderr)
				return
		else:
			print("***(KPdb) Bad arguments: ", arg, file=sys.stderr)


	def do_break_datetime(self, arg, temporary = 0):
		# print >>sys.stdout, "this is KPdb break_datetime"
		# print >>sys.stdout, arg
		try:
			timestamps = arg.split(',')
			for ts in timestamps:
				dt = datetime.utcfromtimestamp(float(ts))
				current_dt = None
				if self.current_time_func is not None:
					current_dt = self.current_time_func()

				if current_dt is None or dt > current_dt:
					DatetimeBreakpoint.datetime_breakpoints[dt] = False
				# print >>sys.stdout, "break point: ",
				# print >>sys.stdout, dt
		except Exception as e:
			print(e, file=sys.stderr)

	def do_unbreak_datetime(self, arg):
		# print >>sys.stdout, "this is KPdb unbreak_datetime"
		# print >>sys.stdout, arg
		try:
			timestamps = arg.split(',')
			for ts in timestamps:
				dt = datetime.utcfromtimestamp(float(ts))
				if dt in DatetimeBreakpoint.datetime_breakpoints:
					del DatetimeBreakpoint.datetime_breakpoints[dt]
		except Exception as e:
			print(e, file=sys.stderr) 

	def _break_at_line(self, filename, lineno, temporary = 0, cond = None):
		funcname = None
		filename = filename.rstrip()
		f = self.lookupmodule(filename)
		if not f:
			return '***(KPdb) ' + repr(filename) + 'not found'
		else:
		    filename = f
		try:
		    lineno = int(lineno)
		except ValueError as msg:
		    return '***(KPdb) Bad lineno:' + str(lineno)

		# now check line
		err = self._checkline(filename, lineno)
		if err:
			return '***(KPdb) ' + err
		# now set the break point
		err = self.set_break(filename, lineno, temporary, cond, funcname)
		if err: 
			return '***(KPdb) ' + err
		# bp = self.get_breaks(filename, line)[-1]
		# print >>self.stdout, "Breakpoint %d at %s:%d" % (bp.number,
		#                                                  bp.file,
		#                                                  bp.line)

	def _break_at_function(self, arg, temporary = 0, cond = None):
		try:
		    func = eval(arg, self.curframe.f_globals, self.curframe_locals)
		except:
		    func = arg
		try:
			if hasattr(func, 'im_func'):
				if PY3:
					func = func.__func__
				else:
					func = func.im_func
			if PY3:
				code = func.__code__
			else:
				code = func.func_code
			#use co_name to identify the bkpt (function names
			#could be aliased, but co_name is invariant)
			funcname = code.co_name
			lineno = code.co_firstlineno
			filename = code.co_filename
		except:
		    # last thing to try
		    (ok, filename, ln) = self.lineinfo(arg)
		    if not ok:
		        return '***(KPdb) The specified object' + repr(arg) + 'is not a function or was not found along sys.path.'

		    funcname = ok # ok contains a function name
		    lineno = int(ln)

		# now check line
		err = self._checkline(filename, lineno)
		if err:
			return '***(KPdb) ' + err
		# now set the break point
		err = self.set_break(filename, lineno, temporary, cond, funcname)
		if err: 
			return '***(KPdb) ' + err

	def _checkline(self, filename, lineno):
		# Check whether specified line seems to be executable.
		globs = self.curframe.f_globals if hasattr(self, 'curframe') and self.curframe is not None else None
		line = linecache.getline(filename, lineno, globs)
		if not line:
		    return '***(KPdb) End of file'
		line = line.strip()
		# Don't allow setting breakpoint at a blank line
		if (not line or (line[0] == '#') or
		     (line[:3] == '"""') or line[:3] == "'''"):
		    return '***(KPdb) Blank or comment'






# -*- coding: utf-8 -*-

from six import with_metaclass
import abc
import sys
import numpy as np
from datetime import datetime, timedelta
from copy import copy

from keystone.performance import risk
from keystone.performance.sample_rate import KSSampleRate
from keystone.riskless import Riskless

class KSAnalyzer(with_metaclass(abc.ABCMeta)):
    '''
    Abstract class of analyzer.
    
    Class properties:
    returnTracker: ReturnTracker instance, shared by all analyzer
    '''
    returnTracker = None
    def getAnnulizedMultiplier(self, squared = True):
        return KSSampleRate.getAnnulizedMultiplier(squared=squared)

    def update(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def value(self, *args, **kwargs):
        pass

class KSReturn(KSAnalyzer):
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoCumulativeReturns) <= 1:
            return 0.0
        return self.returnTracker.algoCumulativeReturns[-1]

class KSBenchmarkReturn(KSAnalyzer):
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.benchmarkCumulativeReturns) <= 1:
            return 0.0
        return self.returnTracker.benchmarkCumulativeReturns[-1]

class KSVolatility(KSAnalyzer):
    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0
        return np.nanstd(self.returnTracker.algoReturns, ddof=1) * self.getAnnulizedMultiplier(squared = True)

class KSBeta(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        http://en.wikipedia.org/wiki/Beta_(finance)
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0
            
        return risk.beta(self.returnTracker.algoReturns, self.returnTracker.benchmarkReturns)

class KSDownsideRisk(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        https://en.wikipedia.org/wiki/Downside_risk
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0
            
        return risk.downside_risk(self.returnTracker.algoReturns,
                             self.returnTracker.algoMeanReturns,
                             self.getAnnulizedMultiplier())

class KSAlpha(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        http://en.wikipedia.org/wiki/Alpha_(investment)
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) == 0:
            return 0.0
            
        beta = KSBeta().value()
        return risk.alpha(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            self.returnTracker.benchmarkAnnulizedMeanReturns[-1],
            beta)

class KSInformationRatio(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        http://en.wikipedia.org/wiki/Information_ratio
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0
            
        volatility = KSVolatility().value()
        return risk.information_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            self.returnTracker.benchmarkAnnulizedMeanReturns[-1],
            volatility)

class KSSortinoRatio(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        http://en.wikipedia.org/wiki/Sortino_ratio
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0

        downsideRisk = KSDownsideRisk().value()
        return risk.sortino_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            downsideRisk)

class KSSharpRatio(KSAnalyzer):
    def value(self, *args, **kwargs):
        """
        http://en.wikipedia.org/wiki/Sharpe_ratio
        """
        if self.returnTracker is None or len(self.returnTracker.algoReturns) <= 1:
            return 0.0

        volatility = KSVolatility().value()
        return risk.sharpe_ratio(
            self.returnTracker.algoAnnulizedMeanReturns[-1],
            Riskless.value,
            volatility)

class KSMaxDrawdown(KSAnalyzer):
    def __init__(self):
        KSAnalyzer.__init__(self)
        self.currentMaxReturn = -np.inf
        self.currentDrawdown = -np.inf

    def value(self, *args, **kwargs):
        if self.returnTracker is None or len(self.returnTracker.algoCumulativeReturns) == 0:
            return 0.0

        self.currentMaxReturn = np.nanmax((self.currentMaxReturn, self.returnTracker.algoCumulativeReturns[-1]))
        # The drawdown is defined as: (high - low) / high
        # The above factors out to: 1.0 - (low / high)
        drawdown = 1.0 - (1.0 + self.returnTracker.algoCumulativeReturns[-1])/(1.0 + self.currentMaxReturn)
        if self.currentDrawdown < drawdown:
            self.currentDrawdown = drawdown

        return self.currentDrawdown


class KSDefaultAnalyzer(KSAnalyzer):
    def __init__(self):
        KSAnalyzer.__init__(self)
        self.benchmark = KSBenchmarkReturn()
        self.returns = KSReturn()
        self.alpha = KSAlpha()
        self.beta = KSBeta()
        self.volatility = KSVolatility()
        self.maxDrawdown = KSMaxDrawdown()
        self.downsideRisk = KSDownsideRisk()
        self.sharpeRatio = KSSharpRatio()
        self.sortinoRatio = KSSortinoRatio()
        self.informationRatio = KSInformationRatio()

    def value(self, *args, **kwargs):
        ret = {
        'return': self.returns.value(),
        'benchmark': self.benchmark.value(),
        'alpha': self.alpha.value(),
        'beta': self.beta.value(),
        'volatility': self.volatility.value(),
        'max_drawdown': self.maxDrawdown.value(),
        'downside_risk': self.downsideRisk.value(),
        'sharpe_ratio': self.sharpeRatio.value(),
        'sortino_ratio': self.sortinoRatio.value(),
        'information_ratio': self.informationRatio.value()
        }
        return ret

class KSCumulativeAnalyzer(KSAnalyzer):
    def __init__(self):
        self.returns = []
        self.benchmark = []
        self.alpha = []
        self.beta = []
        self.volatility = []
        self.max_drawdown = []
        self.downside_risk = []
        self.sharpe_ratio = []
        self.sortino_ratio = []
        self.information_ratio = []

        self.count = 0
        self.defaultAnalyzer = KSDefaultAnalyzer()

    def value(self, *args, **kwargs):
        fields = copy(self.__dict__)
        return fields

    def update(self, *args, **kwargs):
        self.count += 1
        if self.count > 2:
            self.returns.append(self.defaultAnalyzer.returns.value())
            self.benchmark.append(self.defaultAnalyzer.benchmark.value())
            self.alpha.append(self.defaultAnalyzer.alpha.value())
            self.beta.append(self.defaultAnalyzer.beta.value())
            self.volatility.append(self.defaultAnalyzer.volatility.value())
            self.max_drawdown.append(self.defaultAnalyzer.maxDrawdown.value())
            self.downside_risk.append(self.defaultAnalyzer.downsideRisk.value())
            self.sharpe_ratio.append(self.defaultAnalyzer.sharpeRatio.value())
            self.sortino_ratio.append(self.defaultAnalyzer.sortinoRatio.value())
            self.information_ratio.append(self.defaultAnalyzer.informationRatio.value())



# -*- coding: utf-8 -*-

from __future__ import print_function
from keystone.api import keystone_class, api_method
from datetime import datetime
import numpy as np
from keystone.coordinator import KSObserver

class KSUniverse(KSObserver):
    '''
    Universe infomations about security and current backtesting time
    '''
    time = None
    price = {}
    priceUpdateTime = {}
    latestDataEvent = None
    def __init__(self):
        KSObserver.__init__(self)
    
    @classmethod
    def getPriceUpdateTime(cls, security):
        if security in cls.priceUpdateTime:
            return cls.priceUpdateTime[security]
        else:
            return None
        
    @classmethod
    def getPrice(cls, security):
        if security in cls.price:
            return cls.price[security]
        else:
            return np.nan
    
    @classmethod
    def onDataEvent(cls, dataEvent):
        # update time
        cls.time = dataEvent.time()
        
        # update latest dataEvent
        cls.latestDataEvent = dataEvent
        
        # update price
        securities = dataEvent.securities()
        if len(securities) == 0:
            return
        latestPrice = dataEvent.query(securities, 'price')
        if len(securities) == 1:
            cls.price[securities[0]] = latestPrice
            cls.priceUpdateTime[securities[0]] = cls.time
        else:
            cls.price.update(latestPrice.to_dict())
            cls.priceUpdateTime.update({x:cls.time for x in latestPrice.index})


# -*- coding: utf-8 -*-

from functools import wraps
import keystone.api_allow_classes
import inspect

def _get_caller_name(skip=2):
    """Get a name of a caller in the format module.class.method

       `skip` specifies how many levels of stack to skip while getting caller
       name. skip=1 means "who calls me", skip=2 "who calls my caller" etc.

       An empty string is returned if skipped levels exceed stack height
    """
    stack = inspect.stack()
    start = 0 + skip
    if len(stack) < start + 1:
        return ''
    parentframe = stack[start][0]

    name = []
    module = inspect.getmodule(parentframe)
    # `modname` can be None when frame is executed directly in console
    # TODO(techtonik): consider using __main__
    if module:
        name.append(module.__name__)
    # detect classname
    if 'self' in parentframe.f_locals:
        name.append(parentframe.f_locals['self'].__class__.__name__)
    codename = parentframe.f_code.co_name
    if codename != '<module>':  # top level usually
        name.append(codename)  # function or a method
    del parentframe
    return name

def keystone_class(original_class):
    keystone.api_allow_classes.__all__.append('.'.join([original_class.__module__, original_class.__name__]))
    for attr_name, attr_value in original_class.__dict__.items() :
        if not attr_name.startswith('__')  and isinstance(attr_value, types.FunctionType):
            setattr(original_class, attr_name, api_callable_check(attr_value))
    return original_class

def api_callable_check(func):
    '''
    api_callable_check decorator
    WARNNING: api_callable_check MUST BE the top decorator
    '''
    @wraps(func)
    def warpped(*args, **kwargs):
        if hasattr(func, 'is_api_method'):
            return func(*args, **kwargs)
        caller = _get_caller_name()
        caller_module = '.'.join(caller[:-1])
        if caller_module in keystone.api_allow_classes.__all__:
            return func(*args, **kwargs)
        else:
            raise NotImplementedError('function "' + func.__name__ + '" not implemented!')

    return warpped

def api_method(func):
    '''
    api method decorator
    '''
    @wraps(func)
    def warpped(*args, **kwargs):
        return func(*args, **kwargs)

    warpped.is_api_method = 1
    return warpped



# coding=utf-8

from __future__ import print_function
from six import with_metaclass
import abc
from datetime import datetime
import numpy as np

from keystone.api import keystone_class, api_method
from keystone.commission import KSCommissionModel
from keystone.slippage import KSSlippageModel
from keystone.constant import ORDER_ACCEPTED_MESSAGE, ORDER_FILLED_MESSAGE, ORDER_PARTIALLY_FILLED_MESSAGE
from keystone.order import (
                            KSTransaction, 
                            KSOrder,
                            KSOrderState,
                            KSOrderType,
                            KSOrderEvent,
                            KSOrderEventType
                            )
from keystone.universe import KSUniverse as Universe
from keystone.coordinator import KSCoordinator
from keystone.portfolio import KSPortfolio

class KSBrokerPolicy(with_metaclass(abc.ABCMeta)):
    '''
    broker policy
    DO NOT use this class directlly
    '''
    def __init__(self, isInstantMatch = False, slippage = None, commission = None):
        self.isInstantMatch = isInstantMatch
        self.slippage = slippage
        self.commission = commission
        
    def setSlippageModel(self, model):
        if not isinstance(model, KSSlippageModel):
            raise TypeError("slippage model必须是继承自'KSSlippageModel'的实例对象。")
        self.slippage = model
        
    def setCommissionModel(self, model):
        if not isinstance(model, KSCommissionModel):
            raise TypeError("commission model必须是继承自'KSCommissionModel'的实例对象。")
        self.commission = model
        
    def turnOnInstantMatch(self):
        self.isInstantMatch = True
        
    @abc.abstractmethod
    def matchOrder(self, dataEvent, order, *args, **kwargs):
        pass
    
class KSDefaultBrokerPolicy(KSBrokerPolicy):
    def __init__(self):
        KSBrokerPolicy.__init__(self)
        
    def matchOrder(self, dataEvent, order, *args, **kwargs):
        sid = order.securityId()
        shares = order.remaining()
        dealPrice = dataEvent.query(sid, 'price')
        if np.isnan(dealPrice):
            return None
        
        # slippage
        if self.slippage is not None:
            dealPrice = self.slippage.compute(dataEvent, order)

        # commission
        commission = 0
        if self.commission is not None:
            commission = self.commission.compute(dataEvent, order)
        
        return KSTransaction(dataEvent.time(), order.orderid(), sid, shares, dealPrice, commission)
    

class KSBacktestingBroker(object):
    '''
    Backtesting Broker
    '''
    brokerPolicy = KSDefaultBrokerPolicy()

    def __init__(self, *args, **kwargs):
        self.openOrders = {}
        self.filledOrders = {}
        self.cancelledOrders = {}
        self.portfolio = KSPortfolio()
        self.lastEvent = None
    
    def onDataEvent(self, dataEvent):
        self.lastEvent = dataEvent
        self.processAllOpenOrders(dataEvent)
    
    def registerOrder(self, order):
        self.openOrders[order.orderid()] = order
        
    def unregisterOrder(self, order):
        orderid = order.orderid()
        if orderid in self.openOrders:
            if order.state() == KSOrderState.FILLED:
                self.filledOrders[orderid] = order
            elif order.state() == KSOrderState.CANCELLED:
                self.cancelledOrders[orderid] = order
            else:
                pass
            self.openOrders.pop(orderid)
            
    def isExpiredOrder(self, order):
        return order.acceptedAt() < Universe.time
    
    def processAllOpenOrders(self, dataEvent):
        sortedOrders = self.getSortedOpenOrders()
        for order in sortedOrders:
            self.processOrder(dataEvent, order)
            
    def processOrder(self, dataEvent, order): 
        orderid = order.orderid()
        # Step 1 - check order content
        # 如果今天该sid不能交易则返回。例如停牌
        dt = Universe.getPriceUpdateTime(order.sid())
        if  dt is None or dt < Universe.time:
            order.cancel(Universe.time, 'cannot trade for ' + order.sid() + ' in this time.')
            self.unregisterOrder(order)
            self.notifyOrderEvent(orderid, KSOrderEventType.CANCELLED, order.cancelReason(), None)
            return
        
        if order.total() == 0:
            order.cancel(Universe.time, 'cannot place zero quantity order for ' + order.sid() + ' .')
            self.unregisterOrder(order)
            self.notifyOrderEvent(orderid, KSOrderEventType.CANCELLED, order.cancelReason(), None)
            return 
          
        # Step 2 - accept order
        if order.isSubmitted():
            order.accept(Universe.time)
            self.notifyOrderEvent(orderid, KSOrderEventType.ACCEPTED, ORDER_ACCEPTED_MESSAGE, None)
            
        # Step 3 - check and remove expired order
        if self.isExpiredOrder(order):
            order.cancel(Universe.time, 'order expired')
            self.unregisterOrder(order)
            self.notifyOrderEvent(orderid, KSOrderEventType.CANCELLED, order.cancelReason(), None)
            return
        
        # Step 4 - simulating transaction
        txn = self.brokerPolicy.matchOrder(dataEvent, order)
        # txn validation
        if txn is None:
            order.cancel(Universe.time, 'user txn is None, cancel order.')
            self.unregisterOrder(order)
            self.notifyOrderEvent(orderid, KSOrderEventType.CANCELLED, order.cancelReason(), None)
            return
        
        # Step 5 - update order info
        order.update(txn)
        if order.isFilled():
            self.notifyOrderEvent(orderid, KSOrderEventType.FILLED, ORDER_FILLED_MESSAGE, txn)
        else:
            self.notifyOrderEvent(orderid, KSOrderEventType.PARTIALLY_FILLED, ORDER_PARTIALLY_FILLED_MESSAGE, txn)
            
        if order.isCancelled() or order.isFilled():
            self.unregisterOrder(order)
            
    def getSortedOpenOrders(self):
        openOrders = list(self.openOrders.values())
        datetimes = [x.submittedAt() for x in openOrders]
        idx = np.argsort(datetimes)
        return [openOrders[i] for i in idx]
    
    def notifyOrderEvent(self, orderid, type, message, txn):
        orderEvent = KSOrderEvent(orderid, type, message, txn)
        KSCoordinator.dispatchOrderEvent(orderEvent)
        
    def getOrder(self, orderid):
        if orderid in self.openOrders:
            return self.openOrders[orderid]
        elif orderid in self.filledOrders:
            return self.filledOrders[orderid]
        elif orderid in self.cancelledOrders:
            return self.cancelledOrders[orderid]
        else:
            return None
        
    def submitOrder(self, sid, quantity, type = KSOrderType.MARKET_ORDER):
        order = KSOrder(sid, quantity, type)
        order.submit(Universe.time)
        self.registerOrder(order)
        
        if self.brokerPolicy.isInstantMatch:
            self.processOrder(self.lastEvent, order)
            
        return order.orderid()
    
    def order(self, sid, quantity, type = KSOrderType.MARKET_ORDER):
        return self.submitOrder(sid, quantity, type)
    
    def orderValue(self, sid, value, type = KSOrderType.MARKET_ORDER):
        price = Universe.getPrice(sid)
        shares = np.floor(np.abs(value) / price)*np.sign(value)
        return self.submitOrder(sid, shares, type)
    
    def orderPercentage(self, sid, percentage, type = KSOrderType.MARKET_ORDER):
        return self.orderValue(sid, self.portfolio.value() * percentage, type)
    
    def orderTarget(self, sid, targetShares, type = KSOrderType.MARKET_ORDER):
        curShares = 0
        if self.portfolio.hasPosition(sid):
            curShares = self.portfolio.getPosition(sid).quantity()
        return self.submitOrder(sid, targetShares - curShares, type)
        
    def orderTargetValue(self, sid, targetValue, type = KSOrderType.MARKET_ORDER):
        curValue = 0
        if self.portfolio.hasPosition(sid):
            curValue = self.portfolio.getPosition(sid).value()
        return self.orderValue(sid, targetValue - curValue, type)
    
    def orderTargetPercentage(self, sid, percentage, type = KSOrderType.MARKET_ORDER):
        return self.orderTargetValue(sid, self.portfolio.value() * percentage, type)
    
    def cancelOrder(self, orderid):
        if orderid in self.openOrders:
            order = self.openOrders[orderid]
            order.cancel(Universe.time, "cancelled by user")
            self.unregisterOrder(order)


# -*- coding: utf-8 -*-

import math
import numpy as np
from keystone.utils.math_utils import tolerant_equals

############################
# Risk Metric Calculations #
############################


def sharpe_ratio(algorithm_return, treasury_return, algorithm_volatility):
    """
    http://en.wikipedia.org/wiki/Sharpe_ratio
    Args:
        algorithm_volatility (float): Algorithm volatility.
        algorithm_return (float): Algorithm return percentage.
        treasury_return (float): Treasury return percentage.
    Returns:
        float. The Sharpe ratio.
    """
    if tolerant_equals(algorithm_volatility, 0):
        return np.nan

    return (algorithm_return - treasury_return) / algorithm_volatility


def downside_risk(algorithm_returns, mean_returns, normalization_factor):
    rets = np.round(algorithm_returns, 8)
    mar = np.round(mean_returns, 8)
    mask = rets < mar
    downside_diff = rets[mask] - mar[mask]
    if len(downside_diff) <= 1:
        return 0.0
    return np.nanstd(downside_diff, ddof=1) * math.sqrt(normalization_factor)


def sortino_ratio(algorithm_return, treasury_return, mar):
    """
    http://en.wikipedia.org/wiki/Sortino_ratio
    Args:
        algorithm_returns (np.array-like):
            Returns from algorithm lifetime.
        algorithm_period_return (float):
            Algorithm return percentage from latest period.
        mar (float): Minimum acceptable return.
    Returns:
        float. The Sortino ratio.
    """
    if tolerant_equals(mar, 0):
        return np.nan

    return (algorithm_return - treasury_return) / mar


def information_ratio(algorithm_return, benchmark_return, algo_volatility):
    """
    http://en.wikipedia.org/wiki/Information_ratio
    Args:
        algorithm_returns (np.array-like):
            All returns during algorithm lifetime.
        benchmark_returns (np.array-like):
            All benchmark returns during algo lifetime.
    Returns:
        float. Information ratio.
    """
    if tolerant_equals(algo_volatility, 0):
        return np.nan

    # The square of the annualization factor is in the volatility,
    # because the volatility is also annualized,
    # i.e. the sqrt(annual factor) is in the volatility's numerator.
    # So to have the the correct annualization factor for the
    # Sharpe value's numerator, which should be the sqrt(annual factor).
    # The square of the sqrt of the annual factor, i.e. the annual factor
    # itself, is needed in the numerator to factor out the division by
    # its square root.
    return (algorithm_return - benchmark_return) / algo_volatility


def alpha(algorithm_returns, treasury_return,
          benchmark_returns, beta):
    """
    http://en.wikipedia.org/wiki/Alpha_(investment)
    Args:
        algorithm_period_return (float):
            Return percentage from algorithm period.
        treasury_period_return (float):
            Return percentage for treasury period.
        benchmark_period_return (float):
            Return percentage for benchmark period.
        beta (float):
            beta value for the same period as all other values
    Returns:
        float. The alpha of the algorithm.
    """
    return algorithm_returns - \
        (treasury_return + beta *
         (benchmark_returns - treasury_return))

def beta(algorithm_returns, benchmark_returns):
    """
    .. math::
        \\beta_a = \\frac{\mathrm{Cov}(r_a,r_p)}{\mathrm{Var}(r_p)}
    http://en.wikipedia.org/wiki/Beta_(finance)
    """
    # it doesn't make much sense to calculate beta for less than two
    # values, so return none.
    if len(algorithm_returns) < 2:
        return 0.0

    returns_matrix = np.vstack([algorithm_returns,
                                benchmark_returns]).T
    returns_matrix = returns_matrix[~np.isnan(returns_matrix).any(axis=1)].T
    if returns_matrix.shape[1] < 2:
        return 0.0

    C = np.cov(returns_matrix, ddof=1)
    algorithm_covariance = C[0][1]
    benchmark_variance = C[1][1]
    beta = algorithm_covariance / benchmark_variance

    return beta
###########################
# End Risk Metric Section #
###########################


# -*- coding: utf-8 -*-

'''
Session
'''

from __future__ import print_function
import hashlib
import hmac
import logging
import os
import pprint
import random
import uuid
import warnings
import copy
import sys
from datetime import datetime

from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types
from keystone.utils.json_utils import extract_dates, date_default

try:
    import cPickle
    pickle = cPickle
except:
    cPickle = None
    import pickle

try:
    # py3
    PICKLE_PROTOCOL = pickle.DEFAULT_PROTOCOL
except AttributeError:
    PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL

try:
    # We are using compare_digest to limit the surface of timing attacks
    from hmac import compare_digest
except ImportError:
    # Python < 2.7.7: When digests don't match no feedback is provided,
    # limiting the surface of attack
    def compare_digest(a,b): return a == b

import zmq
from zmq.utils import jsonapi
from zmq.eventloop.ioloop import IOLoop
from zmq.eventloop.zmqstream import ZMQStream

json_packer = lambda obj: jsonapi.dumps(obj, default=date_default,
    ensure_ascii=False, allow_nan=False,
)
json_unpacker = lambda s: jsonapi.loads(s)

pickle_packer = lambda o: pickle.dumps(squash_dates(o), PICKLE_PROTOCOL)
pickle_unpacker = pickle.loads

default_packer = json_packer
default_unpacker = json_unpacker

DELIM = b"<IDS|MSG>"

def msg_header(msg_id, msg_type, username, session):
    date = datetime.now()
    return locals()

def session_default():
    u = unicode_type(uuid.uuid4())
    return u

def extract_header(msg_or_header):
    """Given a message or header, return the header."""
    if not msg_or_header:
        return {}
    try:
        # See if msg_or_header is the entire message.
        h = msg_or_header['header']
    except KeyError:
        try:
            # See if msg_or_header is just the header
            h = msg_or_header['msg_id']
        except KeyError:
            raise
        else:
            h = msg_or_header
    if not isinstance(h, dict):
        h = dict(h)
    return h
'''
Session
header: {msg_id: uuid, message id,
		 msg_type: str, type of message,
		 username: str, username,
		 session: uuid, session id,
		 date: str}
parent_header: dict, header of parent
metadata: dict, metadata 
content: dict, content

The Wire Protocol: [idents, DELIM, signature, header, parent_header, metadata, content]
'''
class Session(object):
	pack = default_packer
	unpack = default_unpacker
	session = session_default()
	username = unicode_type(os.environ.get('USER', 'username'))
	auth = None

	def __init__(self, **kwargs):
		self.pid = os.getpid()
		self.bsession = self.session.encode('ascii')
		self.metadata = None
		self.debug = False
		if 'metadata' in kwargs:
			self.metadata = kwargs['metadata']
		if 'debug' in kwargs:
			self.debug = bool(kwargs['debug'])

	@property
	def msg_id(self):
		"""always return new uuid"""
		return str(uuid.uuid4())

	def msg_header(self, msg_type):
	    return msg_header(self.msg_id, msg_type, self.username, self.session)

	def msg(self, msg_type, header=None, parent=None, content=None, metadata=None):
		"""Return the nested message dict.
		This format is different from what is sent over the wire. The
		serialize/deserialize methods converts this nested message dict to the wire
		format, which is a list of message parts.
		"""
		msg = {}
		header = self.msg_header(msg_type) if header is None else header
		msg['header'] = header
		msg['msg_id'] = header['msg_id']
		msg['msg_type'] = header['msg_type']
		msg['content'] = {} if content is None else content
		msg['parent_header'] = {} if parent is None else parent
		msg['metadata'] = copy.copy(self.metadata)
		if metadata is not None:
		    msg['metadata'].update(metadata)
		return msg

	def sign(self, msg_list):
		"""Sign a message with HMAC digest. If no auth, return b''.
		"""
		if self.auth is None:
		    return b''
		h = self.auth.copy()
		for m in msg_list:
		    h.update(m)
		return str_to_bytes(h.hexdigest())

	def serialize(self, msg, ident=None):
		"""Serialize the message components to bytes."""
		content = msg.get('content', {})
		if content is None:
		    content = self.none
		elif isinstance(content, dict):
		    content = json_packer(content)
		elif isinstance(content, bytes):
		    # content is already packed, as in a relayed message
		    pass
		elif isinstance(content, unicode_type):
		    # should be bytes, but JSON often spits out unicode
		    content = json_packer(content)
		else:
		    raise TypeError("Content incorrect type: %s"%type(content))

		real_message = [json_packer(msg['header']),
						json_packer(msg['parent_header']),
		                json_packer(msg['metadata']),
		                content]

		to_send = []

		if isinstance(ident, list):
		    # accept list of idents
		    to_send.extend(ident)
		elif ident is not None:
		    to_send.append(ident)
		to_send.append(DELIM)

		signature = self.sign(real_message)
		to_send.append(signature)

		to_send.extend(real_message)

		return to_send

	def feed_identities(self, msg_list):
		"""Split the identities from the rest of the message.
		"""
		try:
		    idx = msg_list.index(DELIM)
		except:
		    raise ValueError("DELIM not in msg_list")
		return msg_list[:idx], msg_list[idx+1:]

	def deserialize(self, recv_msg_list, content=True):
		"""Unserialize a msg_list to a nested message dict.
		content : bool (True)
		    Whether to unpack the content dict (True), or leave it packed
		    (False).
		"""
		minlen = 5
		idents, msg_list = self.feed_identities(recv_msg_list)
		message = {}
		if not len(msg_list) >= minlen:
		    raise TypeError("malformed message, must have at least %i elements"%minlen)
		header = json_unpacker(msg_list[1])
		message['header'] = extract_dates(header)
		message['msg_id'] = header['msg_id']
		message['msg_type'] = header['msg_type']
		message['parent_header'] = extract_dates(json_unpacker(msg_list[2]))
		message['metadata'] = json_unpacker(msg_list[3])
		if content:
		    message['content'] = json_unpacker(msg_list[4])
		else:
		    message['content'] = msg_list[4]
		if self.debug:
		    pprint.pprint(message)
		return idents, message

	def send(self, stream, msg_type, content=None, header = None, metadata = None, 
		parent=None, ident=None):
		msg = self.msg(msg_type, content=content, parent=parent,
		                   header=header, metadata=metadata)
		to_send = self.serialize(msg, ident)

		stream.send_multipart(to_send, copy=True)

		# print >>sys.stderr,"Session.send():",
		# print >>sys.stderr, to_send
		if self.debug:
		    pprint.pprint(msg)
		    pprint.pprint(to_send)
		    pprint.pprint(buffers)

		return msg

	def recv(self, socket, mode=zmq.NOBLOCK, content=True):
		"""Receive and unpack a message."""
		if isinstance(socket, ZMQStream):
		    socket = socket.socket
		try:
		    msg_list = socket.recv_multipart(mode, copy=True)
		except zmq.ZMQError as e:
		    if e.errno == zmq.EAGAIN:
		        # We can convert EAGAIN to None as we know in this case
		        # recv_multipart won't return None.
		        return None,None
		    else:
		        raise
		try:
		    return self.deserialize(msg_list, content=content)
		except Exception as e:
		    # TODO: handle it
		    raise e




# -*- coding: utf-8 -*-

from __future__ import print_function
import random
import zmq
import sys
import time
import json
import importlib
import imp
import abc
import inspect
import os
from datetime import datetime
from six import with_metaclass

from keystone.strategy import KSStrategy, KSStrategyRunner
from keystone.sources import KSCsvSource
from keystone.performance.analyzer import KSDefaultAnalyzer, KSCumulativeAnalyzer
from keystone.drain import KSDrain
from keystone.app_client.kpdb import KPdb
from keystone.app_client.session import Session
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types
from keystone.utils.json_utils import json_clean

import argparse
parser = argparse.ArgumentParser()
parser.add_argument('--config', help='strategy config', required=True)

def byteify(input):
    if isinstance(input, dict):
        return {byteify(key):byteify(value) for key,value in iteritems(input)}
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif isinstance(input, unicode_type):
        return input.encode('utf-8')
    else:
        return input

def unify(input):
    if isinstance(input, dict):
        return {unify(key):unify(value) for key,value in iteritems(input)}
    elif isinstance(input, list):
        return [unify(element) for element in input]
    elif isinstance(input, string_types):
        return unicode_type(input)
    else:
        return input

def loadUserStrategyClass(filename):
    try:
        strategy = None
        module = imp.load_source('UserStrategy', filename)
        for cls_name in dir(module):
            cls = getattr(module, cls_name)
            if inspect.isclass(cls) and issubclass(cls, KSStrategy) and cls.__name__ != 'KSStrategy':
                if strategy is not None:
                    raise ValueError("find multiple strategy: %s, %s"%(strategy.__name__, cls.__name__))
                else:
                    strategy = cls
    except Exception as e:
        raise ValueError("cannot find load Python code  from %s: %s"%(filename, e))
    if strategy is None:
        raise ValueError("cannot find strategy from %s"%(filename))
    return strategy

'''
StrategyClient

Run backtesting using a config
config format:
{
    "in_url": string, stdin url for ipc, e.g. "tcp://127.0.0.1:55555",
    "drain_url": string, drain url for ipc, e.g. "tcp://127.0.0.1:55556",
    "syn_url": string, syn url for synchronize zmq.publisher socket
    "session": string, uuid,
    "filename": "/Users/rk/Desktop/share_folder/keystone-strategy-engine/node/strategy.py",
    "startingCash": int, 100000, 起始资金
    "startTime": string, 起始时间
    "endTime": string, 结束时间
    "benchmark": dict, {'sid':000001, 'column': 'close'}， 基准
    "riskless": float, 年化无风险利率
    "tradingDays": int, 年交易日
    "tradingHours": int, 日交易时间
    "sampleRate": string, [DAY, HOUR, MINUTE, SECOND]
    "stopPercentage": float, in [0.0,1.0], 止损点
    "historyCapacity": int, history capacity, 要保存的历史长度
    "instanceMatch": bool, 实时撮合
    "sources": [
        {"type": "csv", 文件类型，目前只有csv
         "path": "/Users/rk/Desktop/share_folder/keystone-strategy-engine/data/market.csv", 文件路径
         "dt_column": "dt", 时间列
         "dt_format": "%Y-%m-%d %H:%M:%S", 时间格式
         "price_column": "price", 价格列
         "sid_column": "sid", security code column
         "delimiter": "," csv文件分隔符
        },
        ..., 
        ...]
 }
'''
class StrategyClient(with_metaclass(abc.ABCMeta)):
    # outstream_factory = OutStream
    # outstream_factory.flush_interval = 0.0
    session = Session()
    context = zmq.Context()
    def __init__(self, config):
        if isinstance(config, string_types):
            config = json.loads(config)
        elif isinstance(config, dict):
            config = dict(config)
        else:
            raise TypeError("input config MUST BE 'string' or 'dict'")
        self.config = unify(config)
        self.check_config(config)
        self.strategy_runner = KSStrategyRunner()
        self.init_session()
        self.init_strategy_runner()
        self.init_socket()
        if 'io_redircet' not in config or config['io_redircet'] == True:
            self.io_redirect()

    def check_config(self, config):
        pass

    def run(self):
        self.strategy_runner.run()

    def init_session(self):
        self.session.auth = None
        if 'session' in self.config:
            Session.session = self.config['session']
        self.session_uuid = Session.session

    def init_strategy_runner(self):
        config = self.config
        try:
            # load user strategy class
            userStrategy = loadUserStrategyClass(config['filename'])
            # init source
            sources = config['sources']
            for source_config in sources:
                source = self._get_source(source_config)
                self.strategy_runner.addSource(source)
            # use user strategy class
            self.strategy_runner.setStrategy(userStrategy)
            # benchmark
            if "benchmark" in config:
                benchmark = config["benchmark"]
                if "sid" in benchmark:
                    self.strategy_runner.setBenchmark(benchmark["column"], benchmark["sid"])
                else:
                    self.strategy_runner.setBenchmark(benchmark["column"])
            # instance match
            if "instanceMatch" in config:
                self.strategy_runner.setInstantMatch()
            # history
            if "historyCapacity" in config:
                self.strategy_runner.setHistoryCapacity(config["historyCapacity"])
            # start time and end time
            if "startTime" in config:
                # dt = datetime.strptime(config["startTime"], '%Y-%m-%d %H:%M:%S')
                dt = datetime.utcfromtimestamp(config["startTime"])
                self.strategy_runner.setStartTime(dt)
            if "endTime" in config:
                # dt = datetime.strptime(config["endTime"], '%Y-%m-%d %H:%M:%S')
                dt = datetime.utcfromtimestamp(config['endTime'])
                self.strategy_runner.setEndTime(dt)
            # analyzer
            default_analyzer = KSDefaultAnalyzer()
            self.strategy_runner.attachAnalyzer(default_analyzer)
            # stop percentage
            if "stopPercentage" in config:
                self.strategy_runner.setStopPercentage(config["stopPercentage"])
            # drain address
            if "drain_url" in config and "syn_url" in config:
                self.strategy_runner.setDrainAddress(config["drain_url"], config["syn_url"])
            # riskless
            if "riskless" in config:
                self.strategy_runner.setRiskless(config["riskless"])
            # tradingDays
            if "tradingDays" in config:
                self.strategy_runner.setTradingDays(config["tradingDays"])
            # tradingDays
            if "tradingHours" in config:
                self.strategy_runner.setTradingDays(config["tradingHours"])
            # sampleRate
            if "sampleRate" in config:
                self.strategy_runner.setSampleRate(config["sampleRate"])
        except Exception as e:
            raise ValueError("Initilize strategy config error: %s"%(e))

    def _get_source(self, source_config):
        src_type = source_config['type'].lower()
        if source_config['type'].lower() == "csv":
            source = KSCsvSource(source_config['path'], source_config['dt_column'], source_config['dt_format'])
            # price data or security data
            if 'sid_column' in source_config:
                if 'price_column' in source_config:
                    source.tagAsPriceData(source_config['sid_column'], source_config['price_column'])
                else:
                    source.tagAsSecurityData(source_config['sid_column'])
            # delimiter
            if 'delimiter' in source_config:
                source.setDelimiter(source_config['delimiter'])
            return source
        else:
            raise ValueError("cannot support source type \"%s\""%(src_type))

    def init_socket(self):
        config = self.config
        stdin_socket = self.context.socket(zmq.DEALER)
        # stdin_socket.linger = 1000
        stdin_socket.setsockopt_string(zmq.IDENTITY, self.session_uuid)
        stdin_socket.connect(config['in_url'])
        self.stdin_socket = stdin_socket

        # following code is moved to drain.py
        # stdout_socket = self.context.socket(zmq.PUB)
        # stdout_socket.linger = 1000
        # stdout_socket.bind(config['drain_url'])
        # self.stdout_socket = stdout_socket

    def io_redirect(self):
        self._forward_input()
        # sys.stdout = self.outstream_factory(self.session, self.stdout_socket, u'stdout')
        # sys.stderr = self.outstream_factory(self.session, self.stdout_socket, u'stderr')

    def _forward_input(self):
        if PY3:
            self._sys_raw_input = builtin_mod.input
            builtin_mod.input = self.raw_input
        else:
            self._sys_raw_input = builtin_mod.raw_input
            self._sys_eval_input = builtin_mod.input
            builtin_mod.raw_input = self.raw_input
            builtin_mod.input = lambda prompt='': eval(self.raw_input(prompt))

    def raw_input(self, prompt=''):
        """Forward raw_input to frontends
        Raises
        ------
        StdinNotImplentedError if active frontend doesn't support stdin.
        """
        return self._input_request(prompt, password=False)

    def _input_request(self, prompt, password=False):
        # Flush output before making the request.
        sys.stderr.flush()
        sys.stdout.flush()
        # flush the stdin socket, to purge stale replies
        while True:
            try:
                self.stdin_socket.recv_multipart(zmq.NOBLOCK)
            except zmq.ZMQError as e:
                if e.errno == zmq.EAGAIN:
                    break
                else:
                    raise

        # Send the input request.
        content = json_clean(dict(prompt=prompt, password=password))
        msg = self.session.send(self.stdin_socket, 'input_request', content = content)
        # print("sending input request: " + msg.__str__(), file=sys.stderr)

        # Await a response.
        while True:
            try:
                ident, reply = self.session.recv(self.stdin_socket,0)
                # print("*********ident(%s), recieve %s"%(ident, reply), file=sys.stderr)
            except Exception as e:
                print("Invalid Message", file=sys.stderr)
                raise e
            except KeyboardInterrupt:
                # re-raise KeyboardInterrupt, to truncate traceback
                raise KeyboardInterrupt
            else:
                break
        try:
            # value = reply['content']['value'].encode('utf8', "replace")
            value = reply['content']['value']
        except:
            print("Bad input_reply", file=sys.stderr)
            # self.log.error("Bad input_reply: %s", parent)
            value = ''
        if value == '\x04':
            # EOF
            raise EOFError
        return value

class StrategyRunner(object):
    def run(self, config):
        client = StrategyClient(config)
        client.run()

class StrategyDebugger(object):
    def __init__(self, *args, **kwargs):
        # set datetime break point condition
        # err = self.pdb.do_break_at_function(self.strategy_runner.strategyInstance.onData, 
        #     cond = "self._datetimeBreakPointCls.is_break_at_datetime(data.time())")
        # if err:
        #     print >>sys.stderr, err
        # self.pdb.onecmd('')

        # debug mode also send all data in drain
        KSDrain.send_data = True

    def execute_cmd(self, cmds):
        for cmd in cmds:
            self.pdb.onecmd(cmd)

    def run(self, config):
        # get absolute file path
        config['filename'] = os.path.abspath(config['filename'])
        # init strategy client
        client = StrategyClient(config)        # initialize KPdb
        self.pdb = KPdb(config['filename'],
            client.session, 
            config['notify_url'], 
            config['syn_url'],  
            config['control_url'], 
            datetime_break_func=client.strategy_runner.strategyClass.onData,
            datetime_break_cond="self._datetimeBreakPointCls.is_break_at_datetime(data.time())",
            stdout=sys.stdout);
        self.pdb.use_rawinput = True
        self.pdb.run('client.run()', globals(), locals())





# -*- coding: utf-8 -*-

import numpy as np
from datetime import datetime, timedelta

class KSSampleRate():
    '''
    KSSampleRate store all the informations that need to annulize daily return,
    such as `sample rate`, `trading days` and `trading hours`.
    All of this informations are class properties
    '''

    MONTH = "MONTH"
    DAY = "DAY"
    HOUR = "HOUR"
    MINUTE = "MINUTE"
    SECOND = "SECOND"

    # default rate is DAY
    rate = "DAY"
    tradingDays = 250
    tradingHours = 4
    @classmethod
    def datetimeOffset(cls, rate = None):
        if rate is None:
            rate = cls.rate
        if rate == cls.DAY:
            return timedelta(days = 1)
        elif rate == cls.HOUR:
            return timedelta(hours = 1)
        elif rate == cls.MINUTE:
            return timedelta(minutes = 1)
        elif rate == cls.SECOND:
            return timedelta(seconds = 1)
        else:
            return timedelta(0)

    @classmethod
    def getAnnulizedMultiplier(cls, rate = None, tradingDays = None, tradingHours = None, squared = False):
        if rate is None:
            rate = cls.rate
        if tradingDays is None:
            tradingDays = cls.tradingDays
        if tradingHours is None:
            tradingHours = cls.tradingHours
            
        if rate == KSSampleRate.MONTH:
            multiplier = 12
        elif rate == KSSampleRate.DAY:
            multiplier = tradingDays
        elif rate == KSSampleRate.HOUR:
            multiplier = tradingDays * tradingHours
        elif rate == KSSampleRate.MINUTE:
            multiplier = tradingDays * tradingHours * 60
        elif rate == KSSampleRate.SECOND:
            multiplier = tradingDays * tradingHours * 3600
        else:
            raise ValueError('rate error, rate MUST be ["MONTH", "DAY", "HOUR", "MINUTE", "SECOND"]')
        if squared:
            multiplier = np.sqrt(multiplier)

        return multiplier



# -*- coding: utf-8 -*-

from __future__ import print_function
from datetime import datetime
import numpy as np
from keystone.utils import math_utils
from copy import copy

class KSOrderType(object):
    MARKET_ORDER = 'MARKET_ORDER'
    LIMIT_ORDER = 'LIMIT_ORDER'
    STOP_ORDER = 'STOP_ORDER'
    STOP_LIMIT_ORDER = 'STOP_LIMIT_ORDER'
    
class KSOrderState:
    INITIAL = 'INITIAL'
    SUBMITTED = 'SUBMITTED'
    ACCEPTED = 'ACCEPTED'
    PARTIALLY_FILLED = 'PARTIALLY_FILLED'
    FILLED = 'FILLED'
    CANCELLED = 'CANCELLED'

class KSOrderEventType:
    SUBMITTED = 'SUBMITTED'
    ACCEPTED = 'ACCEPTED'
    PARTIALLY_FILLED = 'PARTIALLY_FILLED'
    FILLED = 'FILLED'
    CANCELLED = 'CANCELLED'


class KSTransaction(object):
    def __init__(self, dt, orderid, sid, quantity, price, commission):
        self.time = dt
        self.orderid = orderid
        self.sid = sid
        self.quantity = quantity
        self.price = price
        self.commission = commission
        
    def to_dict(self):
        fields = copy(self.__dict__)
        return fields
    
    def __repr__(self):
        return self.to_dict().__repr__()
    
    def __str__(self):
        return self.to_dict().__str__()
        

class KSOrder(object):
    def __init__(self, sid, quantity, type = KSOrderType.MARKET_ORDER):
        self.__orderid = math_utils.generate_uuid()
        self.__sid = sid
        self.__quantity = quantity
        self.__type = type
        self.__filled = 0
        self.__avgFilledPrice = 0
        self.__filledPrices = []
        self.__filledQuantities = []
        self.__filledCommissions = []
        self.__state = KSOrderState.INITIAL
        self.__submitTime = None
        self.__acceptTime = None
        self.__updateTime = None
        self.__cancelTime = None
        self.__reason = ''
        
    def __str__(self):
        info = "Order [" + str(self.__orderid) + "]:\n" + \
        "security:\t" + str(self.__sid) + "\n" + \
        "quantity:\t" + str(self.__quantity) + "\n" + \
        "type:\t" + str(self.__type) + "\n" + \
        "filled:\t" + str(self.__filled) + "\n" + \
        "submitTime:\t" + str(self.__submitTime) + "\n" + \
        "acceptTime:\t" + str(self.__acceptTime) + "\n" + \
        "updateTime:\t" + str(self.__updateTime) + "\n" + \
        "cancelTime:\t" + str(self.__cancelTime) + "\n" + \
        "cancelReason:\t" + self.__reason + "\n"
        return info
    
    
    def orderid(self):
        return self.__orderid
    
    
    def sid(self):
        return self.__sid
    
    
    def type(self):
        return self.__type

    
    def state(self):
        return self.__state
    
    
    def securityId(self):
        return self.__sid
    
    
    def total(self):
        return self.__quantity
    
    
    def filled(self):
        return self.__filled
    
    
    def remaining(self):
        return self.__quantity - self.__filled
    
    
    def isSubmitted(self):
        return self.__state == KSOrderState.SUBMITTED
    
    
    def isFilled(self):
        return self.__state == KSOrderState.FILLED
    
    
    def isPratiallyFilled(self):
        return self.__state == KSOrderState.PARTIALLY_FILLED
    
    
    def isCancelled(self):
        return self.__state == KSOrderState.CANCELLED
        
    
    def isSell(self):
        return self.__quantity < 0
    
    
    def submittedAt(self):
        return self.__submitTime
    
    
    def acceptedAt(self):
        return self.__acceptTime
    
    
    def cancelledAt(self):
        return self.__cancelTime
    
    
    def cancelReason(self):
        return self.__reason
    
    
    def avgPrice(self):
        return self.__avgFilledPrice
    
    
    def avgCommission(self):
        return np.mean(self.__filledCommissions)
    
    
    def totalCommission(self):
        return np.sum(self.__filledCommissions)
    
    def setState(self, state):
        self.__state = state
        
    def submit(self, time):
        if self.__state == KSOrderState.INITIAL:
            self.__state = KSOrderState.SUBMITTED
            self.__submitTime = time
        
    def accept(self, time):
        self.__state = KSOrderState.ACCEPTED
        self.__acceptTime = time
    
    def cancel(self, time, reason = ''):
        self.__state = KSOrderState.CANCELLED
        self.__cancelTime = time
        self.__reason = reason
    
    # may raise error, since txn may constructed by user 
    # TODO: 后续支持自定义txn，记得修改错误为中文。   
    def update(self, txn):
        if not isinstance(txn, KSTransaction):
            raise TypeError("txn MUST be a instance of KSTransaction")
        
        if np.sign(txn.quantity) != np.sign(self.__quantity):
            raise TypeError("direction of txn is NOT consistent with this order")
        
        if np.abs(txn.quantity + self.__filled) > np.abs(self.__quantity):
            raise TypeError("txn invalid, np.abs(txn.quantity + self.__filled) > self.__quantity")
        
        self.__filled += txn.quantity
        
        self.__filledPrices.append(txn.price)
        self.__filledQuantities.append(txn.quantity)
        self.__filledCommissions.append(txn.commission)
        
        self.__avgFilledPrice = (np.array(self.__filledPrices) * self.__filledQuantities)/np.sum(self.__filledQuantities)
        
        if self.remaining() == 0:
            self.__state = KSOrderState.FILLED
        else:
            self.__state = KSOrderState.PARTIALLY_FILLED
        
        self.__updateTime = txn.time



class KSOrderEvent(object):
    def __init__(self, orderid, type, message, txn):
        self.orderid = orderid
        self.type = type
        self.message = message
        self.txn = txn
    
    def to_dict(self):
        fields = copy(self.__dict__)
        return fields
    
    def __repr__(self):
        return self.to_dict().__repr__()
    
    def __str__(self):
        return self.to_dict().__str__()
        



# -*- coding: utf-8 -*-

# windows: python setup.py install --boost-dir=../../lib/boost_1.58_win64 --event-engine-include=../../../ --event-engine-lib=../../lib/win64/
# mac: python setup.py install --boost-dir=../../lib/boost_1.58_mac --event-engine-include=../../../ --event-engine-lib=../../lib/mac/
from distutils.version import StrictVersion
from setuptools import (
    Extension,
    find_packages,
    setup,
)

module1 = {'./build/keystone/':['engine.so']}

# setup
setup (name = 'keystone_sdk',
		version = '0.7.1',
		description = 'keystone python SDK for backtesting',
		# package_dir = {'keystone': 'build/keystone'},
		# packages = ['keystone'],
        packages = find_packages('.'),
        package_data = {'keystone': ['engine.cpython-35m-darwin.so', 'engine.so', 'engine.cp35-win_amd64.pyd', 'engine.pyd']},
        install_requires = ['six', 'pandas', 'numpy', 'pyzmq'])
        # data_files = [('keystone',['./test.cpp'])])



# -*- coding: utf-8 -*-

from __future__ import print_function
from keystone.engine import KSEventEngine
import time
import pandas as pd
import sys
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types

class KSBaseSource(dict):
    ''' KSBaseSource for user data '''
    def tagAsPriceData(self, securityColumn, priceColumn):
        if not isinstance(securityColumn, string_types):
            raise TypeError("securityColumn格式错误, securityColumn必须是字符串。")
        if not isinstance(priceColumn, string_types):
            raise TypeError("priceColumn格式错误, priceColumn必须是字符串。")
        self['sid_column'] = securityColumn
        self['price_column'] = priceColumn

    def tagAsSecurityData(self, securityColumn):
        if not isinstance(securityColumn, string_types):
            raise TypeError("securityColumn格式错误, securityColumn必须是字符串。")
        self['sid_column'] = securityColumn

    def setDateFormat(self, dateFormat):
        if not isinstance(dateFormat, string_types):
            raise TypeError("dateFormat格式错误, dateFormat必须是字符串。")
        self['date_format'] = dateFormat

    def setDateColumn(self, dateColumn):
        if not isinstance(dateColumn, string_types):
            raise TypeError("dateColumn格式错误, dateColumn必须是字符串。")
        self['date_column'] = dateColumn

class KSCsvSource(KSBaseSource):
    def __init__(self, 
        filename, 
        dateColumn='datetime', 
        dateFormat="%Y-%m-%d %H:%M:%S", 
        securityColumn=None, 
        priceColumn=None, 
        delimiter=','):
        KSBaseSource.__init__(self)
        self.setDateColumn(dateColumn)
        self.setDateFormat(dateFormat)
        self['type'] = "CSV"
        self['filename'] = filename
        if securityColumn and priceColumn:
            self.tagAsPriceData(securityColumn, priceColumn)
        elif securityColumn:
            self.tagAsSecurityData(securityColumn)
        self.setDelimiter(delimiter)

    def setDelimiter(self, delimiter):
        self['delimiter'] = delimiter
        
class KSMemorySource(KSBaseSource):
    def __init__(self):
        pass
    


# -*- coding: utf-8 -*-

from __future__ import print_function
import sys

class DatetimeBreakpoint(object):
	datetime_breakpoints = {}
	hit_datetime_breakpoint = None

	@classmethod
	def clear(cls):
		cls.datetime_breakpoints = {}
		cls.hit_datetime_breakpoint = None
		
	@classmethod
	def is_break_at_datetime(cls, current_dt):
		sorted_dt = sorted(list(cls.datetime_breakpoints.items()), key=lambda t: t[0])
		break_flag = False
		break_at = None
		for (dt, has_break) in sorted_dt:
			if dt > current_dt:
				break
			if not has_break:
				break_flag = True
				cls.datetime_breakpoints[dt] = True
				break_at = dt
		cls.hit_datetime_breakpoint = break_at
		return break_flag


# -*- coding: utf-8 -*-

from __future__ import print_function
from keystone.utils import math_utils
from keystone.api import keystone_class, api_method
from keystone.order import KSOrderType
from keystone.broker import KSBacktestingBroker
from keystone.portfolio import KSPortfolio
from keystone.history import KSHistory
from keystone.performance.analyzer import KSAnalyzer
from keystone.py3compat import PY3, builtin_mod, iteritems, unicode_type, string_types


class KSContext(object):
    def __init__(self, portfolio, history = None, analyzers = None):
        assert isinstance(portfolio, KSPortfolio)
        self.portfolio = portfolio
        if history is not None:
            assert isinstance(history, KSHistory)
            self.history = history
        if analyzers is not None:
            for analyzer in analyzers:
                assert isinstance(analyzer, KSAnalyzer)
            self.analyzers = analyzers
    
class KSAction(object):
    def __init__(self, broker, logger = None):
        assert isinstance(broker, KSBacktestingBroker)
        self.__broker = broker
        self.__logger = logger
    
    
    def order(self, sid, amount, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isint(amount):
            raise TypeError("amount格式错误，amount必须为整数。")
        
        return self.__broker.order(sid, amount, type)
        
    
    def orderValue(self, sid, value, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isnumber(value):
            raise TypeError("value格式错误，value必须为数字。")
        
        return self.__broker.orderValue(sid, value, type)
        
    
    def orderPercentage(self, sid, percentage, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isnumber(percentage) or abs(percentage) > 1.0:
            raise TypeError("percentage格式错误，percentage必须为数字。")
        
        return self.__broker.orderPercentage(sid, percentage, type)
        
    
    def orderTarget(self, sid, amount, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isint(amount):
            raise TypeError("amount格式错误，amount必须为整数。")
        
        return self.__broker.orderTarget(sid, amount, type)
        
    
    def orderTargetValue(self, sid, value, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isnumber(value):
            raise TypeError("value格式错误，value必须为数字。")
        
        return self.__broker.orderTargetValue(sid, value, type)
        
    
    def orderTargetPercentage(self, sid, percentage, type = KSOrderType.MARKET_ORDER):
        if not isinstance(sid, string_types):
            raise TypeError("sid格式错误，sid必须为字符串。")
        
        if not math_utils.isnumber(percentage) or abs(percentage) > 1.0:
            raise TypeError("percentage格式错误，percentage必须为数字。")
        
        return self.__broker.orderTargetPercentage(sid, percentage, type)
    
    
    def getOrder(self, orderid):
        return self.__broker.getOrder(orderid)